.control.video {
  position: relative;
}
.control.video video {
  width: 800px;
}
.control.video .timestamp {
  position: absolute;
  right: 10px;
  top: 10px;
  background: rgba(0, 0, 0, 0.5);
  border-radius: 5px;
  padding: 5px 10px;
  display: inline-block;
  color: rgba(255, 255, 255, 0.75);
  font-size: 32px;
  font-family: monospace;
}
